
	/* Yacc specification of calculator  */
	/* using ambiguous expression grammar*/
	/* Figure 4.59, Aho, 2nd ed., pg 292 */

%{
#include <stdio.h>
#include <ctype.h>
#define YYSTYPE double  /* double type for Yacc stack */
%}
%token NUMBER

%left '+' '-'
%left '*' '/'
%right UMINUS

%%

lines	: lines expr '\n'	{ printf("%g\n",$2); /* print result */ }
	| lines '\n'		{ ; /* allow blank lines */ }
	| 			{ ; /* empty string */ }
	; 
expr	: expr '+' expr	{ $$ = $1 + $3;}
	| expr '-' expr	{ $$ = $1 - $3;}
	| expr '*' expr	{ $$ = $1 * $3;}
	| expr '/' expr	{ $$ = $1 / $3;}
      	| '(' expr ')'	{ $$ = $2; }
	| '-' expr  %prec UMINUS { $$ = - $2;}
	| NUMBER
	;
%%
int yylex(void)
{ int c;
  while ( (c = getchar()) == ' ' );	/* skip spaces */
  if ( (c == '.') || isdigit(c) ) {
    ungetc(c, stdin);
    scanf("%lf", &yylval);  /* input a double */
    return NUMBER;
  }
  return c;
}

